@@ -15,7 +15,9 @@ from .fireworks import FIREWORKS_MODELS
 from .google import GOOGLE_MODELS
 from .grok import XAI_MODELS
 from .groq import GROQ_MODELS
+from .kimi import KIMI_MODELS
 from .meta import META_MODELS
+from .minimax import MINIMAX_MODELS
 from .mistral import MISTRAL_MODELS
 from .openai import OPENAI_MODELS
 from .openrouter import OPENROUTER_MODELS
@@ -38,9 +40,9 @@ class APIModel:
     supports_responses: bool = False
     reasoning_model: bool = False
     regions: list[str] | dict[str, int] = field(default_factory=list)
-    tokens_per_minute: int | None = None
-    requests_per_minute: int | None = None
-    gpus: list[str] | None = None
+    # tokens_per_minute: int | None = None
+    # requests_per_minute: int | None = None
+    # gpus: list[str] | None = None
 
     @classmethod
     def from_registry(cls, name: str):
@@ -62,7 +64,7 @@ class APIModel:
             raise ValueError("no regions to sample")
         random.sample(regions, 1, counts=weights)[0]
 
-    def make_request(self, context: RequestContext):  # -> "APIRequestBase"
+    def make_request(self, context: RequestContext):
         from ..api_requests.common import CLASSES
 
         api_spec = self.api_spec
@@ -97,8 +99,8 @@ def register_model(
     supports_responses: bool = False,
     reasoning_model: bool = False,
     regions: list[str] | dict[str, int] = field(default_factory=list),
-    tokens_per_minute: int | None = None,
-    requests_per_minute: int | None = None,
+    # tokens_per_minute: int | None = None,
+    # requests_per_minute: int | None = None,
 ) -> APIModel:
     """Register a model configuration and return the created APIModel."""
     model = APIModel(
@@ -116,8 +118,8 @@ def register_model(
         supports_responses=supports_responses,
         reasoning_model=reasoning_model,
         regions=regions,
-        tokens_per_minute=tokens_per_minute,
-        requests_per_minute=requests_per_minute,
+        # tokens_per_minute=tokens_per_minute,
+        # requests_per_minute=requests_per_minute,
     )
     registry[model.id] = model
     return model
@@ -132,7 +134,9 @@ for model_dict in [
     FIREWORKS_MODELS,
     GOOGLE_MODELS,
     XAI_MODELS,
+    KIMI_MODELS,
     META_MODELS,
+    MINIMAX_MODELS,
     MISTRAL_MODELS,
     OPENAI_MODELS,
     OPENROUTER_MODELS,
